@@ -1,16 +1,18 @@
 //! `copy` subcommand
 
 use crate::{
-    commands::open_repository, helpers::table_with_titles, status_err, Application, RUSTIC_APP,
+    commands::{get_repository, init::init_password, open_repository, open_repository_indexed},
+    config::AllRepositoryOptions,
+    helpers::table_with_titles,
+    status_err, Application, RUSTIC_APP,
 };
 use abscissa_core::{Command, Runnable, Shutdown};
 use anyhow::{bail, Result};
 use log::{error, info};
-
 use merge::Merge;
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 
-use rustic_core::{CopySnapshot, Id, KeyOptions, Repository, RepositoryOptions};
+use rustic_core::{CopySnapshot, Id, KeyOptions};
 
 /// `copy` subcommand
 #[derive(clap::Parser, Command, Debug)]
@@ -29,11 +31,11 @@ pub(crate) struct CopyCmd {
 }
 
 /// Target repository options
-#[derive(Default, Clone, Debug, Deserialize, Merge)]
+#[derive(Default, Clone, Debug, Serialize, Deserialize, Merge)]
 pub struct Targets {
     /// Target repositories
     #[merge(strategy = merge::vec::overwrite_empty)]
-    targets: Vec<RepositoryOptions>,
+    targets: Vec<AllRepositoryOptions>,
 }
 
 impl Runnable for CopyCmd {
@@ -54,7 +56,7 @@ impl CopyCmd {
             RUSTIC_APP.shutdown(Shutdown::Crash);
         }
 
-        let repo = open_repository(&config)?.to_indexed()?;
+        let repo = open_repository_indexed(&config.repository)?;
         let mut snapshots = if self.ids.is_empty() {
             repo.get_matching_snapshots(|sn| config.snapshot_filter.matches(sn))?
         } else {
@@ -65,8 +67,9 @@ impl CopyCmd {
 
         let poly = repo.config().poly()?;
         for target_opt in &config.copy.targets {
-            let repo_dest = Repository::new(target_opt)?;
+            let repo_dest = get_repository(target_opt)?;
 
+            info!("copying to target {}...", repo_dest.name);
             let repo_dest = if self.init && repo_dest.config_id()?.is_none() {
                 if config.global.dry_run {
                     error!(
@@ -77,13 +80,12 @@ impl CopyCmd {
                 }
                 let mut config_dest = repo.config().clone();
                 config_dest.id = Id::random();
-                let pass = repo_dest.password()?.unwrap();
+                let pass = init_password(&repo_dest)?;
                 repo_dest.init_with_config(&pass, &self.key_opts, config_dest)?
             } else {
-                repo_dest.open()?
+                open_repository(target_opt)?
             };
 
-            info!("copying to target {}...", repo_dest.name);
             if poly != repo_dest.config().poly()? {
                 bail!("cannot copy to repository with different chunker parameter (re-chunking not implemented)!");
             }
