@@ -1,7 +1,8 @@
 //! `restore` subcommand
 
 use crate::{
-    commands::open_repository, helpers::bytes_size_to_string, status_err, Application, RUSTIC_APP,
+    commands::open_repository_indexed, helpers::bytes_size_to_string, status_err, Application,
+    RUSTIC_APP,
 };
 
 use abscissa_core::{Command, Runnable, Shutdown};
@@ -52,7 +53,7 @@ impl RestoreCmd {
     fn inner_run(&self) -> Result<()> {
         let config = RUSTIC_APP.config();
         let dry_run = config.global.dry_run;
-        let repo = open_repository(&config.repository)?.to_indexed()?;
+        let repo = open_repository_indexed(&config.repository)?;
 
         let node =
             repo.node_from_snapshot_path(&self.snap, |sn| config.snapshot_filter.matches(sn))?;
